Package: localboot
Type: Package
Title: Local Bootstrap Methods for Various Networks
Version: 0.9.2
Date: 2024-01-08
Authors@R: c(
    person("Tianhai", "Zu", email = "zuti@mail.uc.edu", role = c("aut", "cre")),
    person("Yichen", "Qin", email = "qinyn@ucmail.uc.edu", role = c("aut","ctb"))
  )
Description: Network analysis usually requires estimating the uncertainty of
    graph statistics. Through this package, we provide tools to bootstrap 
    various networks via local bootstrap procedure. Additionally, it includes 
    functions for generating probability matrices, creating network adjacency 
    matrices from probability matrices, and plotting network structures. 
    The reference will be updated soon.
License: GPL-3
Depends: R (>= 3.5.0)
Imports: graphics, viridis, Rcpp (>= 1.0.11)
Suggests: testthat (>= 3.0.0)
LinkingTo: Rcpp (>= 1.0.11), RcppEigen
Encoding: UTF-8
RoxygenNote: 7.2.3
NeedsCompilation: yes
Packaged: 2024-01-09 23:36:45 UTC; tianhaizu
Author: Tianhai Zu [aut, cre],
  Yichen Qin [aut, ctb]
Maintainer: Tianhai Zu <zuti@mail.uc.edu>
Repository: CRAN
Date/Publication: 2024-01-10 11:33:21 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-28 19:01:21 UTC; unix
Archs: localboot.so.dSYM
