# Generated by roxygen2: do not edit by hand

export(generate_graphon)
export(generate_network_P)
export(localboot)
export(plot_P)
export(plot_adj)
importFrom(Rcpp,evalCpp)
importFrom(graphics,filled.contour)
importFrom(viridis,viridis)
useDynLib(localboot)
